@@ -39,6 +39,7 @@ char *opids[] =
    /* Simplifying */
    PCL_EVALGC,
    PCL_RW,
+   PCL_LOCAL_RW,
    PCL_RW,
    PCL_AD,
    PCL_CSR,
@@ -63,12 +64,17 @@ char *opids[] =
    PCL_OF,
    PCL_EF,
    PCL_ER,
+   PCL_DDC,
    PCL_SAT,
+   PCL_PE_RESOLVE,
    /* Others */
    PCL_SE,
    PCL_ID_DEF,
    PCL_SC,
+   PCL_EQ_TO_EQ,
+   PCL_LL,
    PCL_FU,
+   PCL_LIFT_ITE,
    PCL_EBV,
    /* HO inferences */
    PCL_DYN_CNF,
@@ -78,7 +84,14 @@ char *opids[] =
    PCL_POS_EXT,
    PCL_EXT_SUP,
    PCL_EXT_EQRES,
-   PCL_INV_REC
+   PCL_EXT_EQFACT,
+   PCL_INV_REC,
+   PCL_CHOICE_AX,
+   PCL_LEIBNIZ_ELIM,
+   PCL_PRIM_ENUM,
+   PCL_CHOICE_INST,
+   PCL_TRIGGER,
+   PCL_PRUNE_ARG
 };
 
 char *optheory [] =
@@ -97,6 +110,7 @@ char *optheory [] =
    NULL,
    NULL,
    NULL,
+   NULL,
    "answers",
    /* Simplification/Modfication for FOF */
    NULL,
@@ -114,12 +128,17 @@ char *optheory [] =
    NULL,
    NULL,
    NULL,
+   NULL,
+   NULL,
    /* Others */
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
+   NULL,
+   NULL,
+   NULL,
    /* HO inferences */
    NULL,
    NULL,
@@ -128,6 +147,13 @@ char *optheory [] =
    NULL,
    NULL,
    NULL,
+   NULL,
+   NULL,
+   NULL,
+   NULL,
+   NULL,
+   NULL,
+   NULL,
    NULL
 };
 
@@ -149,6 +175,7 @@ char *opstatus [] =
    "thm",
    "thm",
    "thm",
+   "thm",
    /* Simplification/Modfication for FOF */
    "cth",
    "thm",
@@ -165,12 +192,17 @@ char *opstatus [] =
    "thm",
    "thm",
    "thm",
+   "thm",
+   "thm",
    /* Others */
    "thm",
    NULL,
    "thm",
    "thm",
    "thm",
+   "thm",
+   "thm",
+   "thm",
    /* HO */
    "thm",
    "thm",
@@ -179,6 +211,13 @@ char *opstatus [] =
    "thm",
    "thm",
    "thm",
+   "thm",
+   "thm",
+   "thm",
+   "thm",
+   "thm",
+   "thm",
+   "thm",
    "thm"
 };
 
@@ -481,7 +520,7 @@ bool DerivedInProof(Derived_p derived)
 
 /*-----------------------------------------------------------------------
 //
-// Function: DerivedMarkProof()
+// Function: DerivedSetInProof()
 //
 //   Mark a derived cell as a proof cell.
 //
@@ -518,6 +557,32 @@ void DerivedSetInProof(Derived_p derived, bool in_proof)
 }
 
 
+/*-----------------------------------------------------------------------
+//
+// Function: DerivedCollectFCodes()
+//
+//   Collect all f_codes from the logical clause/formula into
+//   *tree. Return number of new entries found.
+//
+// Global Variables: -
+//
+// Side Effects    : -
+//
+/----------------------------------------------------------------------*/
+
+long DerivedCollectFCodes(Derived_p derived, NumTree_p *tree)
+{
+   if(derived->clause)
+   {
+      return ClauseCollectFCodes(derived->clause, tree);
+   }
+   else
+   {
+      return TermCollectFCodes(derived->formula->tformula, tree);
+   }
+}
+
+
 
 /*-----------------------------------------------------------------------
 //
@@ -1037,6 +1102,65 @@ Derived_p DerivedAlloc(void)
 }
 
 
+/*-----------------------------------------------------------------------
+//
+// Function: DerivationStackPCLPrint()
+//
+//   Print a very short description of the derivation for debug
+//   purposes.
+//
+// Global Variables: -
+//
+// Side Effects    : Output
+//
+/----------------------------------------------------------------------*/
+
+void DerivationDebugPrint(FILE* out, PStack_p derivation)
+{
+   if(derivation)
+   {
+      PStackPointer sp = PStackGetSP(derivation);
+      PStackPointer i  = 0;
+      DerivationCode op;
+      while(i<sp)
+      {
+         op = PStackElementInt(derivation, i);
+         i++;
+         if(DCOpHasArg1(op))
+         {
+            if(DCOpHasCnfArg1(op))
+            {
+               DBG_PRINT(out, "[", ClausePrintDBG(out, PStackElementP(derivation, i)), "]");
+            }
+            else if(DCOpHasFofArg1(op))
+            {
+               WFormula_p f = PStackElementP(derivation, i);
+               DBG_PRINT(out, "[", WFormulaTSTPPrint(stderr, f, true, true), "]");
+            }
+            i++;
+         }
+         if(DCOpHasArg2(op))
+         {
+            if(DCOpHasCnfArg2(op))
+            {
+               DBG_PRINT(out, "[", ClausePrintDBG(out, PStackElementP(derivation, i)), "]");
+            }
+            else if(DCOpHasFofArg2(op))
+            {
+               WFormula_p f = PStackElementP(derivation, i);
+               DBG_PRINT(out, "[", WFormulaTSTPPrint(stderr, f, true, true), "]");
+            }
+            i++;
+         }
+         fprintf(out, "<%s%s>", opids[DPOpGetOpCode(op)], i==sp?"":",");
+      }
+   }
+   else
+   {
+      fprintf(out, " - ");
+   }
+}
+
 
 /*-----------------------------------------------------------------------
 //
@@ -2248,6 +2372,35 @@ void DerivationAnalyse(Derivation_p derivation)
    }
 }
 
+/*-----------------------------------------------------------------------
+//
+// Function: DerivationCollectFCodes()
+//
+//   Collect all f_codes from derivation into tree.
+//
+// Global Variables: -
+//
+// Side Effects    : -
+//
+/----------------------------------------------------------------------*/
+
+long DerivationCollectFCodes(Derivation_p derivation, NumTree_p *tree)
+{
+   PStackPointer sp;
+   Derived_p     handle;
+   long          res = 0;
+
+   if(!derivation->ordered)
+   {
+      DerivationTopoSort(derivation);
+   }
+   for(sp=0; sp<PStackGetSP(derivation->ordered_deriv); sp++)
+   {
+      handle = PStackElementP(derivation->ordered_deriv, sp);
+      res += DerivedCollectFCodes(handle, tree);
+   }
+   return res;
+}
 
 
 /*-----------------------------------------------------------------------
@@ -2262,14 +2415,13 @@ void DerivationAnalyse(Derivation_p derivation)
 //
 /----------------------------------------------------------------------*/
 
-void DerivationPrint(FILE* out, Derivation_p derivation, char* frame)
+void DerivationPrint(FILE* out, Derivation_p derivation)
 {
    PStackPointer sp;
    Derived_p     node;
 
    assert(derivation->ordered);
 
-   fprintf(out, "# SZS output start %s\n", frame);
    for(sp=PStackGetSP(derivation->ordered_deriv)-1; sp>=0; sp--)
    {
       node = PStackElementP(derivation->ordered_deriv, sp);
@@ -2287,7 +2439,6 @@ void DerivationPrint(FILE* out, Derivation_p derivation, char* frame)
       }
       fprintf(out, "\n");
    }
-   fprintf(out, "# SZS output end %s\n", frame);
 }
 
 
@@ -2355,7 +2506,21 @@ void DerivationPrintConditional(FILE* out, char* status, Derivation_p derivation
 {
    if(print_derivation == POList)
    {
-      DerivationPrint(GlobalOut, derivation, status);
+      fprintf(out, "# SZS output start %s\n", status);
+      if(sig->typed_symbols)
+      {
+         NumTree_p symbols = NULL;
+         PTree_p types = NULL;
+
+         DerivationCollectFCodes(derivation, &symbols);
+         SigFCodesCollectTypes(sig, symbols, &types);
+         TypeBankPrintSelectedSortDefs(out, sig->type_bank, types);
+         SigPrintTypeDeclsTSTPSelective(out, sig, &symbols);
+         NumTreeFree(symbols);
+         PTreeFree(types);
+      }
+      DerivationPrint(GlobalOut, derivation);
+      fprintf(out, "# SZS output end %s\n", status);
    }
    else if(print_derivation >= POGraph1)
    {
