@@ -222,6 +222,7 @@ ProofState_p ProofStateAlloc(FunctionProperties free_symb_prop)
    handle->factor_count         = 0;
    handle->neg_ext_count        = 0;
    handle->resolv_count         = 0;
+   handle->disequ_deco_count    = 0;
    handle->satcheck_count       = 0;
    handle->satcheck_success     = 0;
    handle->satcheck_satisfiable = 0;
@@ -389,7 +390,6 @@ void ProofStateResetClauseSets(ProofState_p state, bool term_gc)
    if(term_gc)
    {
       TBGCCollect(state->terms);
-      //GCCollect(state->gc_original_terms);
    }
 }
 
@@ -634,8 +634,16 @@ void ProofStateStatisticsPrint(FILE* out, ProofState_p state)
            state->neg_ext_count);
    fprintf(out, "# Equation resolutions                 : %ld\n",
            state->resolv_count);
+   fprintf(out, "# Disequality decompositions           : %ld\n",
+           state->disequ_deco_count);
    fprintf(out, "# Total rewrite steps                  : %lu\n",
            state->rw_count);
+   /* RewriteUncached strictly counts possible rw-steps put into the
+      cache. In some rare situations, proof search can terminate
+      before they are realised, leading to more uncached steps than
+      actual steps. We correct that here. */
+   fprintf(out, "# ...of those cached                   : %lu\n",
+           MAX(0,(long)state->rw_count-(long)RewriteUncached));
    fprintf(out, "# Propositional unsat checks           : %ld\n",
            state->satcheck_count);
    fprintf(out, "#    Propositional check models        : %ld\n",
