@@ -29,6 +29,7 @@
 #include <ccl_def_handling.h>
 #include <ccl_garbage_coll.h>
 #include <ccl_global_indices.h>
+#include <ccl_rewrite.h>
 #include <picosat.h>
 
 /*---------------------------------------------------------------------*/
@@ -97,6 +98,7 @@ typedef struct proofstatecell
    unsigned long factor_count;
    unsigned long neg_ext_count;
    unsigned long resolv_count;
+   unsigned long disequ_deco_count;
    unsigned long satcheck_count;
    unsigned long satcheck_success;
    unsigned long satcheck_satisfiable;
