@@ -8,14 +8,12 @@
 
   Functions for CNF conversion of a term-encoded FOF formula.
 
-  Copyright 2003-2019 by the author.
+  Copyright 2003-2024 by the author(s).
   This code is released under the GNU General Public Licence and
   the GNU Lesser General Public License.
   See the file COPYING in the main E directory for details..
   Run "eprover -h" for contact information.
 
-  Changes
-
   Created: Thu Feb 26 00:21:17 CET 2004
 
   -----------------------------------------------------------------------*/
@@ -254,14 +252,14 @@ TFormula_p troot_nnf(TB_p terms, TFormula_p form, int polarity)
          }
          else if(TermIsTrueTerm(form->args[0]))
          {
-            handle = TFormulaFCodeAlloc(terms, terms->sig->eqn_code,
+            handle = TFormulaFCodeAlloc(terms, terms->sig->neqn_code,
                                         form->args[0],
                                         form->args[0]);
 
          }
          else if(TermIsFalseTerm(form->args[0]))
          {
-            handle = TFormulaFCodeAlloc(terms, terms->sig->neqn_code,
+            handle = TFormulaFCodeAlloc(terms, terms->sig->eqn_code,
                                         form->args[0],
                                         form->args[0]);
 
@@ -1766,6 +1764,9 @@ TFormula_p TFormulaExpandLiterals(TB_p terms, TFormula_p form)
          form = form->args[0];
       }
    }
+   /* printf("TFormulaExpandLiterals end ["); */
+   /* TermPrettyPrintSimple(stdout, form, terms->sig, 0); */
+   /* printf(" ]\n"); */
    return form;
 }
 
