@@ -230,6 +230,9 @@ void HeuristicParmsInitialize(HeuristicParms_p handle)
    handle->split_aggressive              = false;
    handle->split_fresh_defs              = true;
 
+   handle->diseq_decomposition           = 0;
+   handle->diseq_decomp_maxarity         = LONG_MAX;
+
    strcpy(handle->rw_bw_index_type, DEFAULT_RW_BW_INDEX_NAME);
    strcpy(handle->pm_from_index_type, DEFAULT_PM_FROM_INDEX_NAME);
    strcpy(handle->pm_into_index_type, DEFAULT_PM_INTO_INDEX_NAME);
@@ -435,6 +438,9 @@ void HeuristicParmsPrint(FILE* out, HeuristicParms_p handle)
    fprintf(out, "   split_fresh_defs:               %s\n",
            BOOL2STR(handle->split_fresh_defs));
 
+   fprintf(out, "   diseq_decomposition             %ld\n", handle->diseq_decomposition);
+   fprintf(out, "   diseq_decomp_maxarity           %ld\n", handle->diseq_decomp_maxarity);
+
    fprintf(out, "   rw_bw_index_type:               %s\n", handle->rw_bw_index_type);
    fprintf(out, "   pm_from_index_type:             %s\n", handle->pm_from_index_type);
    fprintf(out, "   pm_into_index_type:             %s\n", handle->pm_into_index_type);
@@ -674,6 +680,9 @@ bool HeuristicParmsParseInto(Scanner_p in,
    PARSE_INT_LIMITED(split_method, 0, 2);
    PARSE_BOOL(split_aggressive);
    PARSE_BOOL(split_fresh_defs);
+   PARSE_INT(diseq_decomposition);
+   PARSE_INT(diseq_decomp_maxarity);
+
    PARSE_IDENT_INTO(rw_bw_index_type, MAX_PM_INDEX_NAME_LEN);
    PARSE_IDENT_INTO(pm_from_index_type, MAX_PM_INDEX_NAME_LEN);
    PARSE_IDENT_INTO(pm_into_index_type, MAX_PM_INDEX_NAME_LEN);
