@@ -132,6 +132,8 @@ typedef enum
    OPT_SPLIT_HOW,
    OPT_SPLIT_AGGRESSIVE,
    OPT_SPLIT_REUSE_DEFS,
+   OPT_DISEQ_DECOMPOSITION,
+   OPT_DISEQ_DECOMP_MAXARITY,
    OPT_ORDERING,
    OPT_TO_WEIGHTGEN,
    OPT_TO_WEIGHTS,
@@ -297,16 +299,11 @@ OptCell opts[] =
     "positions, and level 3 will expand this to a proof object where "
     "all intermediate results are explicit. This feature is under "
     "development, so far only level 0 and 1 are operational. "
-    "By default The proof object will be provided in TPTP-3 or LOP "
+    "The proof object will be provided in TPTP-3 or PCL "
     "syntax, depending on input format and explicit settings. The "
-    "following option will suppress normal output of the proof "
+    "--proof-graph option will suppress normal output of the proof "
     "object in favour of a graphial representation."},
 
-   {OPT_PROOF_STATS,
-    '\0', "proof-statistics",
-    NoArg, NULL,
-    "Print various statistics of the proof object."},
-
    {OPT_PROOF_GRAPH,
     '\0', "proof-graph",
     OptArg, "3",
@@ -317,6 +314,11 @@ OptCell opts[] =
     "labelled with the TPTP clause/formula) or 3  (nodes "
     "also labelled with source/inference record."},
 
+   {OPT_PROOF_STATS,
+    '\0', "proof-statistics",
+    NoArg, NULL,
+    "Print various statistics of the proof object."},
+
    {OPT_FULL_DERIV,
     'd', "full-deriv",
     NoArg, NULL,
@@ -377,7 +379,7 @@ OptCell opts[] =
     NoArg, NULL,
     "Print data about the proof state that is potentially expensive "
     "to collect. Includes number of term cells and number of "
-    "rewrite steps."},
+    "rewrite steps. This implies the previous option."},
 
    {OPT_PRINT_SATURATED,
     'S', "print-saturated",
@@ -480,17 +482,13 @@ OptCell opts[] =
    {OPT_CPU_LIMIT,
     '\0', "cpu-limit",
     OptArg, "300",
-    "Limit the cpu time the prover should run. The optional argument "
-    "is the CPU time in seconds. The prover will terminate immediately"
-    " after reaching the time limit, regardless of internal state. This"
-    " option may not work "
-    "everywhere, due to broken and/or strange behaviour of setrlimit() "
-    "in some UNIX implementations. It does work under all tested "
-    "versions of Solaris, HP-UX, MacOS-X, and GNU/Linux. As a side "
-    "effect, this "
-    "option will inhibit core file writing. Please note that if you"
-    " use both --cpu-limit and --soft-cpu-limit, the soft limit has to"
-    " be smaller than the hard limit to have any effect. "},
+    "Limit the (per core) cpu time the prover should run. The optional "
+    "argument is the CPU time in seconds. The prover will terminate "
+    "immediately after reaching the time limit, regardless of internal "
+    "state. As a side effect, this option will inhibit core file "
+    "writing. Please note that if you use both --cpu-limit and "
+    "--soft-cpu-limit, the soft limit has to"
+    "be smaller than the hard limit to have any effect. "},
 
    {OPT_SOFTCPU_LIMIT,
     '\0', "soft-cpu-limit",
@@ -516,9 +514,11 @@ OptCell opts[] =
 
    {OPT_PRINT_STRATEGY,
     '\0', "print-strategy",
-    NoArg, NULL,
-    "Print a representation of all search parameters and their setting. "
-    "Then terminate."},
+    OptArg, ">current-strategy<",
+    "Print a representation of all search parameters and their setting "
+    "of a given strategy, then terminate. If no argument is given, "
+    "the current strategy is printed. Use the reserved name '>all-strats<'"
+    "to get a description of all built-in strategies."},
 
    {OPT_PARSE_STRATEGY,
     '\0', "parse-strategy",
@@ -640,9 +640,9 @@ OptCell opts[] =
    {OPT_TSTP_PARSE,
     '\0', "tstp-in",
     NoArg, NULL,
-    "Set TPTP-3 as the input format (Note that TPTP-3 syntax "
-    "is still under development, and the version in E may not be "
-    "fully conforming at all times. E works on all TPTP 6.3.0 FOF "
+    "Set TPTP-3 as the input format TPTP-3 syntax is still under "
+    "development, and any given version in E may not be "
+    "fully conforming at all times. E works on all TPTP 8.2.0 FOF "
     "and CNF files (including includes)."},
 
    {OPT_TSTP_PRINT,
@@ -682,11 +682,12 @@ OptCell opts[] =
     "Use the (experimental) strategy scheduling. This will try several "
     "different fully specified search strategies (aka \"Auto-Modes\"), "
     "one after the other, until a proof or saturation is found, or the "
-    "time limit is exceeded. Optional argument is the number of CPUs "
+    "time limit is exceeded. The optional argument is the number of CPUs "
     "on which the schedule is going to be executed on. By default, "
     "the schedule is executed on a single core. To execute on all "
-    "cores of a system set the argument to 'Auto', but note that this"
-    "will use all reported cores (even efficiency cores, if available)."},
+    "cores of a system, set the argument to 'Auto', but note that this "
+    "will use all reported cores (even low-performance efficiency "
+    "cores, if available on the hardware platform and reported by the OS)."},
 
     {OPT_FORCE_PREPROC_SCHED,
     '\0', "force-preproc-sched",
@@ -1037,6 +1038,19 @@ OptCell opts[] =
     NoArg, NULL,
     "If possible, reuse previous definitions for splitting."},
 
+   {OPT_DISEQ_DECOMPOSITION,
+    '\0', "disequality-decomposition",
+    OptArg, "1024",
+    "Enable the disequality decomposition inference. The optional "
+    "argument is the maximal literal number of clauses considered "
+    "for the inference."},
+
+   {OPT_DISEQ_DECOMP_MAXARITY,
+    '\0', "disequality-decomp-maxarity",
+    OptArg, "1",
+    "Limit disequality decomposition to function symbols of at most "
+    "the given arity."},
+
    {OPT_ORDERING,
     't', "term-ordering",
     ReqArg, NULL,
