@@ -8,7 +8,7 @@
 
   Main program for the E equational theorem prover.
 
-  Copyright 1998-2021 by the authors.
+  Copyright 1998-2024 by the authors.
   This code is released under the GNU General Public Licence and
   the GNU Lesser General Public License.
   See the file COPYING in the main E directory for details..
@@ -57,6 +57,7 @@ PERF_CTR_DEFINE(SatTimer);
 char              *outname = NULL;
 char              *watchlist_filename = NULL;
 char              *parse_strategy_filename = NULL;
+char              *print_strategy = NULL;
 HeuristicParms_p  h_parms;
 FVIndexParms_p    fvi_parms;
 bool              print_sat = false,
@@ -65,7 +66,6 @@ bool              print_sat = false,
    proof_statistics = false,
    filter_sat = false,
    print_rusage = false,
-   print_strategy = false,
    print_pid = false,
    print_version = false,
    outinfo = false,
@@ -294,7 +294,18 @@ void strategy_io(HeuristicParms_p h_parms, PStack_p hcb_definitions)
 
    if(print_strategy)
    {
-      HeuristicParmsPrint(stdout, h_parms);
+      if(strcmp(print_strategy, ">all-strats<")==0)
+      {
+         StrategiesPrintPredefined(GlobalOut);
+      }
+      else
+      {
+         if(strcmp(print_strategy, ">current-strategy<")!=0)
+         {
+            GetHeuristicWithName(print_strategy, h_parms);
+         }
+         HeuristicParmsPrint(GlobalOut, h_parms);
+      }
       exit(NO_ERROR);
    }
 }
@@ -448,6 +459,16 @@ static void print_proof_stats(ProofState_p proofstate,
 #endif
       fprintf(GlobalOut, "# Termbank termtop insertions          : %lld\n",
               proofstate->terms->insertions);
+      fprintf(GlobalOut, "# Search garbage collected termcells   : %lld\n",
+              proofstate->terms->recovered);
+      if(TBPrintDetails)
+      {
+         TBGCCollect(proofstate->terms);
+         fprintf(GlobalOut, "# Final garbage collected termcells    : %lld\n",
+                 proofstate->terms->recovered);
+         fprintf(GlobalOut, "# Final shared term nodes              : %ld\n",
+                 TBTermNodes(proofstate->terms));
+      }
       PERF_CTR_PRINT(GlobalOut, MguTimer);
       PERF_CTR_PRINT(GlobalOut, SatTimer);
       PERF_CTR_PRINT(GlobalOut, ParamodTimer);
@@ -1235,6 +1256,7 @@ CLState_p process_options(int argc, char* argv[])
             break;
       case OPT_EXPENSIVE_DETAILS:
             TBPrintDetails = true;
+            print_statistics = true;
             break;
       case OPT_PRINT_SATURATED:
             outdesc = arg;
@@ -1325,7 +1347,7 @@ CLState_p process_options(int argc, char* argv[])
             print_rusage = true;
             break;
       case OPT_PRINT_STRATEGY:
-            print_strategy = true;
+            print_strategy = arg;
             break;
       case OPT_PARSE_STRATEGY:
             parse_strategy_filename = arg;
@@ -1614,6 +1636,12 @@ CLState_p process_options(int argc, char* argv[])
       case OPT_SPLIT_REUSE_DEFS:
             h_parms->split_fresh_defs = false;
             break;
+      case OPT_DISEQ_DECOMPOSITION:
+            h_parms->diseq_decomposition = CLStateGetIntArg(handle, arg);
+            break;
+      case OPT_DISEQ_DECOMP_MAXARITY:
+            h_parms->diseq_decomp_maxarity = CLStateGetIntArg(handle, arg);
+            break;
       case OPT_ORDERING:
             if(strcmp(arg, "LPO")==0)
             {
