@@ -30,6 +30,7 @@
 #include "cte_typecheck.h"
 #include <cte_typebanks.h>
 #include <ccl_tformulae.h>
+#include <cte_lambda.h>
 
 
 
@@ -80,7 +81,7 @@ static void tb_print_dag(FILE *out, NumTree_p in_index, Sig_p sig)
    term = in_index->val1.p_val;
    fprintf(out, "*%ld : ", term->entry_no);
 
-   if(TermIsVar(term))
+   if(TermIsFreeVar(term))
    {
       VarPrint(out, term->f_code);
    }
@@ -133,8 +134,13 @@ static Term_p tb_termtop_insert(TB_p bank, Term_p t)
    Term_p new;
 
    assert(t);
-   assert(!TermIsVar(t));
-   assert(!TermIsAppliedVar(t) || TermIsVar(t->args[0]));
+   assert(!TermIsAnyVar(t));
+   assert(t->f_code != SIG_NAMED_LAMBDA_CODE ||
+          (t->arity == 2 && TermIsFreeVar(t->args[0])));
+   assert(t->f_code != SIG_DB_LAMBDA_CODE ||
+          (t->arity == 2 && TermIsDBVar(t->args[0])));
+   assert(!TermIsPhonyApp(t) || TermIsAnyVar(t->args[0]) || TermIsLambda(t->args[0]));
+   assert(!TermIsPhonyApp(t) || t->arity > 1);
 
 #ifndef NDEBUG
    for(int i=0; i<t->arity; i++)
@@ -158,25 +164,61 @@ static Term_p tb_termtop_insert(TB_p bank, Term_p t)
       assert(!TermIsShared(t));
       new->properties = (new->properties | t->properties)/*& bank->prop_mask*/;
       TermTopFree(t);
+      TermSetBank(t, bank);
       t = new;
    }
    else
    {
       t->entry_no     = ++(bank->in_count);
+      TermSetBank(t, bank);
       TermCellAssignProp(t,TPGarbageFlag, bank->garbage_state);
       TermCellSetProp(t, TPIsShared); /* Groundness may change below */
+      if(TermIsDBVar(t))
+      {
+         TermCellSetProp(t, TPHasDBSubterm);
+      }
+      if(TypeIsBool(t->type))
+      {
+         TermCellSetProp(t, TPHasBoolSubterm);
+      }
       if(TermIsPhonyApp(t) && TermIsLambda(t->args[0]))
       {
          TermCellSetProp(t, TPIsBetaReducible);
       }
+      if(TermIsLambda(t))
+      {
+         TermCellSetProp(t, TPHasLambdaSubterm);
+      }
+      if(TypeIsArrow(t->type) && !TermIsLambda(t))
+      {
+         TermCellSetProp(t, TPHasEtaExpandableSubterm);
+      }
+      if(t->f_code == bank->sig->eqn_code || t->f_code == bank->sig->neqn_code)
+      {
+         TermCellSetProp(t, TPHasEqNeqSym);
+      }
       t->v_count = 0;
-      t->f_count = !TermIsAppliedVar(t) ? 1 : 0;
+      t->f_count = !TermIsPhonyApp(t) ? 1 : 0;
       t->weight = DEFAULT_FWEIGHT*t->f_count;
       for(int i=0; i<t->arity; i++)
       {
-         assert(TermIsShared(t->args[i])||TermIsVar(t->args[i]));
+         assert(TermIsShared(t->args[i])||TermIsFreeVar(t->args[i]));
          TermCellSetProp(t, TermCellGiveProps(t->args[i], TPIsBetaReducible));
-         if(TermIsVar(t->args[i]))
+         TermCellSetProp(t, TermCellGiveProps(t->args[i], TPHasDBSubterm));
+         TermCellSetProp(t, TermCellGiveProps(t->args[i], TPHasEqNeqSym));
+         TermCellSetProp(t, TermCellGiveProps(t->args[i], TPHasBoolSubterm));
+         if(TypeIsBool(t->args[i]))
+         {
+            TermCellSetProp(t, TPHasBoolSubterm); // vars and dbvars are sometimes not shared.
+         }
+         TermCellSetProp(t, TermCellGiveProps(t->args[i], TPHasLambdaSubterm));
+         if(!(TermIsPhonyApp(t) || TermIsLambda(t)) || i!=0)
+         {
+            TermCellSetProp(t, TermCellGiveProps(t->args[i], TPHasEtaExpandableSubterm));
+         }
+         TermCellSetProp(t, TermCellGiveProps(t->args[i], TPHasNonPatternVar));
+         TermCellSetProp(t, TermCellGiveProps(t->args[i], TPHasAppVar));
+         if(TermIsFreeVar(t->args[i]))
          {
             t->v_count += 1;
             t->weight  += DEFAULT_VWEIGHT;
@@ -189,6 +231,27 @@ static Term_p tb_termtop_insert(TB_p bank, Term_p t)
          }
       }
 
+      if(t->f_code == SIG_DB_LAMBDA_CODE)
+      {
+         t->f_count -= 2; // adjusting for db var and lambda sign
+         t->weight  -= 2*DEFAULT_FWEIGHT; // adjusting  for db var and lambda sign
+      }
+      if(TermIsAppliedFreeVar(t))
+      {
+         TermCellSetProp(t, TPHasAppVar);
+         // counting applied pattern free var as a single var.
+         if(NormalizePatternAppVar(bank, t))
+         {
+            t->f_count = 0;
+            t->v_count = 1;
+            t->weight  = DEFAULT_VWEIGHT;
+         }
+         else
+         {
+            TermCellSetProp(t, TPHasNonPatternVar);
+         }
+      }
+
       if(t->v_count == 0)
       {
          TermCellSetProp(t, TPIsGround);
@@ -200,7 +263,7 @@ static Term_p tb_termtop_insert(TB_p bank, Term_p t)
       //assert(TermIsGround(t) == TermIsGroundCompute(t));
    }
 
-   TermSetBank(t, bank);
+
    return t;
 }
 
@@ -376,8 +439,7 @@ static Term_p parse_let_sym_def(Scanner_p in, TB_p bank, PStack_p type_decls)
       AcceptInpTok(in, Colon);
       AcceptInpTok(in, EqualSign);
 
-      Term_p rhs = TypeIsPredicate(type) ?
-                     TFormulaTSTPParse(in, bank) : TBTermParse(in, bank);
+      Term_p rhs = TFormulaTSTPParse(in, bank);
       Term_p lhs = TermTopAlloc(id, arity);
       for(int i=0; i<arity; i++)
       {
@@ -439,14 +501,14 @@ static Term_p tb_subterm_parse(Scanner_p in, TB_p bank)
 {
    Term_p res = TBTermParseReal(in, bank, true);
 
-   if(!TermIsVar(res))
+   if(!TermIsFreeVar(res))
    {
       if(SigIsPredicate(bank->sig, res->f_code))
       {
          if(SigIsFixedType(bank->sig, res->f_code))
          {
             AktTokenError(in,
-                          "Predicate used as function symbol in preceeding term",
+                          "Predicate used as function symbol in preceding term",
                           false);
          }
          else
@@ -495,6 +557,68 @@ static Term_p choose_subterm_parse_fun(bool check_symb_prop,
    return res;
 }
 
+/*-----------------------------------------------------------------------
+//
+// Function: normalize_boolean_terms()
+//
+//   If term_ref points to an equation of type X=true that appears
+//   under context, replace this equation by X.
+//
+// Global Variables: -
+//
+// Side Effects    : -
+//
+/----------------------------------------------------------------------*/
+Term_p do_tb_insert_no_props_cached(TB_p bank, Term_p term,
+                                    DerefType deref, PObjMap_p* cache)
+{
+   int    i;
+   Term_p t;
+
+   assert(term);
+
+   term = problemType == PROBLEM_HO && deref == DEREF_ALWAYS ?
+            WHNF_deref(term) : TermDeref(term, &deref);
+   Term_p cached = PObjMapFind(cache, term, PCmpFun);
+   if(cached)
+   {
+      t = cached;
+   }
+   else
+   {
+      const int limit = DEREF_LIMIT(term, deref);
+
+      if(TermIsFreeVar(term))
+      {
+         t = VarBankVarAssertAlloc(bank->vars, term->f_code, term->type);
+         TermSetBank(t, bank);
+      }
+      else if (TermIsDBVar(term))
+      {
+         t = TBRequestDBVar(bank, term->type, term->f_code);
+         TermSetBank(t, bank);
+      }
+      else
+      {
+         t = TermTopCopyWithoutArgs(term); /* This is an unshared term cell at the moment */
+         t->properties = TPIgnoreProps;
+
+         assert(SysDateIsCreationDate(t->rw_data.nf_date[0]));
+         assert(SysDateIsCreationDate(t->rw_data.nf_date[1]));
+
+         for(i=0; i<t->arity; i++)
+         {
+            t->args[i] = do_tb_insert_no_props_cached(bank, term->args[i],
+                                                      CONVERT_DEREF(i, limit, deref), cache);
+         }
+         t = tb_termtop_insert(bank, t);
+      }
+      PObjMapStore(cache, term, t, PCmpFun);
+   }
+   return t;
+}
+
+
 /*-----------------------------------------------------------------------
 //
 // Function: normalize_boolean_terms()
@@ -515,7 +639,7 @@ static void normalize_boolean_terms(Term_p* term_ref, TB_p bank)
    Sig_p  sig  = bank->sig;
    if(term->f_code == sig->eqn_code)
    {
-      if(TermIsVar(term->args[0]) && term->args[1]->f_code == SIG_TRUE_CODE)
+      if(TermIsFreeVar(term->args[0]) && term->args[1]->f_code == SIG_TRUE_CODE)
       {
          *term_ref = term->args[0]; // garbage collection will deal with parent
       }
@@ -552,7 +676,7 @@ static void normalize_boolean_terms(Term_p* term_ref, TB_p bank)
 /----------------------------------------------------------------------*/
 
 static Term_p tb_term_parse_arglist(Scanner_p in, TB_p bank,
-                                       bool check_symb_prop, Type_p type)
+                                    bool check_symb_prop, Type_p type)
 {
    Term_p   tmp;
    Term_p result;
@@ -624,11 +748,14 @@ TB_p TBAlloc(Sig_p sig)
 
    handle->in_count = 0;
    handle->insertions = 0;
-   handle->rewrite_steps = 0;
+   handle->recovered = 0;
    handle->ext_index = PDIntArrayAlloc(1,100000);
    handle->garbage_state = TPIgnoreProps;
+   handle->gc = GCAdminAlloc();
    handle->sig = sig;
    handle->vars = VarBankAlloc(sig->type_bank);
+   handle->vars->term_bank = handle;
+   handle->db_vars = DBVarBankAlloc();
    TermCellStoreInit(&(handle->term_store));
 
    term = TermConstCellAlloc(SIG_TRUE_CODE);
@@ -668,8 +795,10 @@ void TBFree(TB_p junk)
     */
    TermCellStoreExit(&(junk->term_store));
    PDArrayFree(junk->ext_index);
+   GCAdminFree(junk->gc);
    VarBankFree(junk->vars);
    PDArrayFree(junk->min_terms);
+   DBVarBankFree(junk->db_vars);
    //assert(!junk->freevarsets);
    TBCellFree(junk);
 }
@@ -738,12 +867,16 @@ Term_p TBInsert(TB_p bank, Term_p term, DerefType deref)
 
    assert(term);
 
-   term = TermDeref(term, &deref);
+   term = problemType == PROBLEM_HO && deref == DEREF_ALWAYS ?
+            WHNF_deref(term) : TermDeref(term, &deref);
 
-   if(TermIsVar(term))
+   if(TermIsFreeVar(term))
    {
       t = VarBankVarAssertAlloc(bank->vars, term->f_code, term->type);
-      TermSetBank(t, bank);
+   }
+   else if (TermIsDBVar(term))
+   {
+      t = TBRequestDBVar(bank, term->type, term->f_code);
    }
    else
    {
@@ -762,12 +895,58 @@ Term_p TBInsert(TB_p bank, Term_p term, DerefType deref)
    return t;
 }
 
+/*-----------------------------------------------------------------------
+//
+// Function: TBInsertInstantiatedDeref()
+//
+//  Insert the term, following the bindings of the variables
+//  according to DerefType.
+//
+// Global Variables: -
+//
+// Side Effects    : Changes term bank
+//
+/----------------------------------------------------------------------*/
+
+Term_p TBInsertInstantiatedDeref(TB_p bank, Term_p term, DerefType deref)
+{
+   if(deref == DEREF_NEVER)
+   {
+      return term;
+   }
+   int    i;
+   Term_p t;
+   const int limit = DEREF_LIMIT(term, deref);
+
+   assert(term);
+   term = problemType == PROBLEM_HO && deref == DEREF_ALWAYS ?
+            WHNF_deref(term) : TermDeref(term, &deref);
+
+   if(TermIsAnyVar(term) || TermIsGround(term))
+   {
+      t = term;
+   }
+   else
+   {
+      t = TermTopCopyWithoutArgs(term); /* This is an unshared term cell at the moment */
+
+      for(i=0; i<t->arity; i++)
+      {
+         t->args[i] =
+            TBInsertInstantiatedDeref(bank ? bank : TermGetBank(term),
+                                      term->args[i], CONVERT_DEREF(i, limit, deref));
+      }
+      t = tb_termtop_insert(bank, t);
+   }
+   return t;
+}
+
 /*-----------------------------------------------------------------------
 //
 // Function: TBInsertIgnoreVar()
 //
 //  As TBInsert, but does instead of using variables from the term bank,
-//  uses the ones already present in the temr. 
+//  uses the ones already present in the temr.
 //
 //  TermProperties are masked with bank->prop_mask.
 //
@@ -785,9 +964,10 @@ Term_p TBInsertIgnoreVar(TB_p bank, Term_p term, DerefType deref)
 
    assert(term);
 
-   term = TermDeref(term, &deref);
+   term = problemType == PROBLEM_HO && deref == DEREF_ALWAYS ?
+            WHNF_deref(term) : TermDeref(term, &deref);
 
-   if(TermIsVar(term))
+   if(TermIsAnyVar(term))
    {
       return term;
    }
@@ -826,16 +1006,25 @@ Term_p TBInsertNoProps(TB_p bank, Term_p term, DerefType deref)
 {
    int    i;
    Term_p t;
+   TB_p tmp_bank = TermGetBank(term);
 
    assert(term);
 
+   TermSetBank(term, bank); //Cheat because WHNF_deref() needs it, God
+                            //knows for what!
    const int limit = DEREF_LIMIT(term, deref);
-   term = TermDeref(term, &deref);
+   t = problemType == PROBLEM_HO && deref == DEREF_ALWAYS ?
+      WHNF_deref(term) : TermDeref(term, &deref);
+   TermSetBank(term, tmp_bank);
+   term = t;
 
-   if(TermIsVar(term))
+   if(TermIsFreeVar(term))
    {
       t = VarBankVarAssertAlloc(bank->vars, term->f_code, term->type);
-      TermSetBank(t, bank);
+   }
+   else if (TermIsDBVar(term))
+   {
+      t = TBRequestDBVar(bank, term->type, term->f_code);
    }
    else
    {
@@ -855,6 +1044,35 @@ Term_p TBInsertNoProps(TB_p bank, Term_p term, DerefType deref)
    return t;
 }
 
+/*-----------------------------------------------------------------------
+//
+// Function: TBInsertNoPropsCached()
+//
+//   As TBInsert, but will set all properties of the new term to 0
+//   first. Also, use a cache so that work is not repeated for the
+//   same terms.
+//
+// Global Variables: TBSupportReplace
+//
+// Side Effects    : Changes term bank
+//
+/----------------------------------------------------------------------*/
+#define CACHE_THRESHOLD 8096
+Term_p TBInsertNoPropsCached(TB_p bank, Term_p term, DerefType deref)
+{
+   Term_p res;
+   if(term->f_count > CACHE_THRESHOLD)
+   {
+      PObjMap_p cache = NULL;
+      res = do_tb_insert_no_props_cached(bank, term, deref, &cache);
+      PObjMapFree(cache);
+   }
+   else
+   {
+      res = TBInsertNoProps(bank, term, deref);
+   }
+   return res;
+}
 
 /*-----------------------------------------------------------------------
 //
@@ -884,12 +1102,16 @@ Term_p  TBInsertRepl(TB_p bank, Term_p term, DerefType deref, Term_p old, Term_p
    }
 
    const int limit = DEREF_LIMIT(term, deref);
-   term = TermDeref(term, &deref);
+   term = problemType == PROBLEM_HO && deref == DEREF_ALWAYS ?
+            WHNF_deref(term) : TermDeref(term, &deref);
 
-   if(TermIsVar(term))
+   if(TermIsFreeVar(term))
    {
       t = VarBankVarAssertAlloc(bank->vars, term->f_code, term->type);
-      TermSetBank(t, bank);
+   }
+   else if (TermIsDBVar(term))
+   {
+      t = TBRequestDBVar(bank, term->type, term->f_code);
    }
    else
    {
@@ -938,7 +1160,7 @@ Term_p TBInsertReplPlain(TB_p bank, Term_p term, Term_p old, Term_p repl)
    {
       return term;
    }
-   if(TermIsVar(term))
+   if(TermIsAnyVar(term))
    {
       return term;
    }
@@ -1011,10 +1233,13 @@ Term_p TBInsertInstantiatedFO(TB_p bank, Term_p term)
       return term->binding;
    }
 
-   if(TermIsVar(term))
+   if(TermIsFreeVar(term))
    {
       t = VarBankVarAssertAlloc(bank->vars, term->f_code, term->type);
-      TermSetBank(t, bank);
+   }
+   else if (TermIsDBVar(term))
+   {
+      t = TBRequestDBVar(bank, term->type, term->f_code);
    }
    else
    {
@@ -1059,47 +1284,34 @@ Term_p TBInsertInstantiatedHO(TB_p bank, Term_p term, bool follow_bind)
       return term;
    }
 
-   if(TermIsVar(term) && term->binding)
+   if(TermIsFreeVar(term) && term->binding)
    {
       t = follow_bind ? TBInsert(bank, term->binding, DEREF_NEVER) : term;
       TermSetBank(t, bank);
       return t;
    }
+   else if (TermIsDBVar(term))
+   {
+      t = TBRequestDBVar(bank, term->type, term->f_code);
+      return t;
+   }
 
    int ignore_args = 0;
-   if(TermIsAppliedVar(term) && term->args[0]->binding && follow_bind)
+   if(TermIsAppliedFreeVar(term) && term->args[0]->binding && follow_bind)
    {
       Term_p binding = term->args[0]->binding;
       ignore_args =
-         TermIsLambda(binding) ? 1 : (binding->arity + (TermIsVar(binding) ? 1 : 0));
+         TermIsLambda(binding) ? 1 : (binding->arity + (TermIsFreeVar(binding) ? 1 : 0));
       DerefType d = DEREF_ONCE;
       term = TermDeref(term, &d);
    }
 
-   if(TermIsVar(term))
+   if(TermIsFreeVar(term))
    {
       t = VarBankVarAssertAlloc(bank->vars, term->f_code, term->type);
-      TermSetBank(t, bank);
    }
    else
    {
-      Term_p binding_copy = NULL;
-      if(TFormulaIsQuantified(bank->sig, term))
-      {
-         if(term->args[0]->binding)
-         {
-            // making sure to avoid capture
-            binding_copy = term->args[0]->binding;
-            term->args[0]->binding = NULL;
-         }
-         else
-         {
-            term->args[0]->binding = VarBankGetFreshVar(bank->vars,
-                                                        term->args[0]->type);
-         }
-
-      }
-
       t = TermTopCopyWithoutArgs(term); /* This is an unshared term cell at the moment */
       t->properties    = TPIgnoreProps;
 
@@ -1111,16 +1323,10 @@ Term_p TBInsertInstantiatedHO(TB_p bank, Term_p term, bool follow_bind)
          t->args[i] = TBInsertInstantiatedHO(bank, term->args[i], follow_bind && (i >= ignore_args));
       }
       t = tb_termtop_insert(bank, t);
-
-      if(TFormulaIsQuantified(bank->sig, term))
-      {
-         term->args[0]->binding = binding_copy;
-      }
    }
    return t;
 }
 
-#ifdef ENABLE_LFHO
 /*-----------------------------------------------------------------------
 //
 // Function: TBInsertInstantiated()
@@ -1145,7 +1351,6 @@ __inline__ Term_p TBInsertInstantiated(TB_p bank, Term_p term)
       return TBInsertInstantiatedFO(bank, term);
    }
 }
-#endif
 
 
 /*-----------------------------------------------------------------------
@@ -1170,7 +1375,8 @@ Term_p TBInsertOpt(TB_p bank, Term_p term, DerefType deref)
    assert(term);
 
    const int limit = DEREF_LIMIT(term, deref);
-   term = TermDeref(term, &deref);
+   term = problemType == PROBLEM_HO && deref == DEREF_ALWAYS ?
+            WHNF_deref(term) : TermDeref(term, &deref);
 
    if(TermIsGround(term))
    {
@@ -1178,10 +1384,13 @@ Term_p TBInsertOpt(TB_p bank, Term_p term, DerefType deref)
       return term;
    }
 
-   if(TermIsVar(term))
+   if(TermIsFreeVar(term))
    {
       t = VarBankVarAssertAlloc(bank->vars, term->f_code, term->type);
-      TermSetBank(t, bank);
+   }
+   else if (TermIsDBVar(term))
+   {
+      t = TBRequestDBVar(bank, term->type, term->f_code);
    }
    else
    {
@@ -1227,12 +1436,15 @@ Term_p  TBInsertDisjoint(TB_p bank, Term_p term)
       return term;
    }
 
-   if(TermIsVar(term))
+   if(TermIsFreeVar(term))
    {
       t = VarBankGetAltVar(bank->vars, term);
-      TermSetBank(t, bank);
       // t = VarBankVarAssertAlloc(bank->vars, term->f_code+1, term->sort);
    }
+   else if (TermIsDBVar(term))
+   {
+      t = TBRequestDBVar(bank, term->type, term->f_code);
+   }
    else
    {
       t = TermTopCopyWithoutArgs(term); /* This is an unshared term cell at the moment */
@@ -1309,10 +1521,14 @@ Term_p TBAllocNewSkolem(TB_p bank, PStack_p variables, Type_p ret_type)
 
 Term_p TBFind(TB_p bank, Term_p term)
 {
-   if(TermIsVar(term))
+   if(TermIsFreeVar(term))
    {
       return VarBankFCodeFind(bank->vars, term->f_code);
    }
+   else if (TermIsDBVar(term))
+   {
+      return TBRequestDBVar(bank, term->type, term->f_code);
+   }
    return TermCellStoreFind(&(bank->term_store), term);
 }
 
@@ -1378,7 +1594,7 @@ void TBPrintTermCompact(FILE* out, TB_p bank, Term_p term)
    }
    else
    {
-      if(TermIsVar(term))
+      if(TermIsFreeVar(term))
       {
          VarPrint(out, term->f_code);
       }
@@ -1498,7 +1714,7 @@ Term_p TBTermParseReal(Scanner_p in, TB_p bank, bool check_symb_prop)
    StreamType    type_stream;
 
    source_name = DStrGetRef(AktToken(in)->source);
-   type_stream        = AktToken(in)->stream_type;
+   type_stream = AktToken(in)->stream_type;
    line = AktToken(in)->line;
    column = AktToken(in)->column;
 
@@ -1606,7 +1822,7 @@ Term_p TBTermParseReal(Scanner_p in, TB_p bank, bool check_symb_prop)
             DStrFree(errpos);
          }
          handle = tb_termtop_insert(bank, handle);
-         }
+      }
       DStrFree(id);
    }
    DStrReleaseRef(source_name);
@@ -1614,6 +1830,119 @@ Term_p TBTermParseReal(Scanner_p in, TB_p bank, bool check_symb_prop)
    return handle;
 }
 
+/*-----------------------------------------------------------------------
+//
+// Function: TBTermParseSimple()
+//
+//   Parses terms without giving any special semantics to symbols.
+//   Input variant of TermPrintSimple().
+//
+// Global Variables: -
+//
+// Side Effects    : Input, memory operations, changes termbank.
+//
+/----------------------------------------------------------------------*/
+
+Term_p TBTermParseSimple(Scanner_p in, TB_p bank)
+{
+   Term_p        handle;
+   DStr_p        id;
+   FuncSymbType  id_type;
+   DStr_p        source_name, errpos;
+   Type_p        type;
+   long          line, column;
+   StreamType    type_stream;
+
+   source_name = DStrGetRef(AktToken(in)->source);
+   type_stream = AktToken(in)->stream_type;
+   line = AktToken(in)->line;
+   column = AktToken(in)->column;
+
+   /* Normal term stuff, bloated because of the nonsensical SETHEO
+      syntax */
+   id = DStrAlloc();
+
+   if((id_type=TermParseOperator(in, id))==FSIdentVar)
+   {
+      /* A variable may be annotated with a sort */
+      if(TestInpTok(in, Colon))
+      {
+         AcceptInpTok(in, Colon);
+         type = TypeBankParseType(in, bank->sig->type_bank);
+         handle = VarBankExtNameAssertAllocSort(bank->vars,
+                                                DStrView(id), type);
+      }
+      else
+      {
+         handle = VarBankExtNameAssertAlloc(bank->vars, DStrView(id));
+      }
+   }
+   else
+   {
+      if(TestInpTok(in, OpenBracket))
+      {
+         AcceptInpTok(in, OpenBracket);
+         if(TestInpTok(in, CloseBracket))
+         {
+            NextToken(in);
+            handle = TermDefaultCellAlloc();
+         }
+         else
+         {
+            PStack_p args = PStackAlloc();
+            int i=0;
+
+            PStackPushP(args, TBTermParseSimple(in, bank));
+            i++;
+
+            while(TestInpTok(in, Comma))
+            {
+               NextToken(in);
+               PStackPushP(args, TBTermParseSimple(in, bank));
+               i++;
+            }
+
+            AcceptInpTok(in, CloseBracket);
+            int arity = PStackGetSP(args);
+            handle = TermDefaultCellArityAlloc(arity);
+
+            for(i=0;i<arity;i++)
+            {
+               handle->args[i] = PStackElementP(args,i);
+            }
+
+            PStackFree(args);
+         }
+      }
+      else
+      {
+         handle = TermDefaultCellAlloc();
+         handle->arity = 0;
+      }
+      handle->f_code = TermSigInsert(bank->sig, DStrView(id),
+                                       handle->arity, false, id_type);
+      if(!handle->f_code)
+      {
+         errpos = DStrAlloc();
+         DStrAppendStr(errpos, PosRep(type_stream, source_name, line, column));
+         DStrAppendStr(errpos, DStrView(id));
+         DStrAppendStr(errpos, " used with arity ");
+         DStrAppendInt(errpos, (long)handle->arity);
+         DStrAppendStr(errpos, ", but registered with arity ");
+         DStrAppendInt(errpos,
+                        (long)(bank->sig)->
+                        f_info[SigFindFCode(bank->sig, DStrView(id))].arity);
+         Error(DStrView(errpos), SYNTAX_ERROR);
+         DStrFree(errpos);
+      }
+      handle = tb_termtop_insert(bank, handle);
+   }
+   DStrFree(id);
+   DStrReleaseRef(source_name);
+
+   return handle;
+}
+
 /*-----------------------------------------------------------------------
 //
 // Function: TBRefSetProp()
@@ -1631,10 +1960,10 @@ void TBRefSetProp(TB_p bank, TermRef ref, TermProperties prop)
 {
    Term_p term, new;
 
-   assert(!TermIsVar(*ref));
+   assert(!TermIsAnyVar(*ref));
 
    term = *ref;
-   if(TermCellQueryProp(term, prop)||TermIsVar(term))
+   if(TermCellQueryProp(term, prop)||TermIsAnyVar(term))
    {
       return;
    }
@@ -1666,7 +1995,7 @@ void TBRefDelProp(TB_p bank, TermRef ref, TermProperties prop)
    Term_p term, new;
 
    term = *ref;
-   if(!TermCellIsAnyPropSet(term, prop)||TermIsVar(term))
+   if(!TermCellIsAnyPropSet(term, prop)||TermIsAnyVar(term))
    {
       return;
    }
@@ -1752,7 +2081,7 @@ void TBGCMarkTerm(TB_p bank, Term_p term)
             PStackPushP(stack, TermRWReplaceField(term));
          }
 
-         if(TermIsAppliedVar(term) && TermGetCache(term))
+         if(TermIsAppliedFreeVar(term) && TermGetCache(term))
          {
             PStackPushP(stack, TermGetCache(term));
          }
@@ -1810,7 +2139,7 @@ long TBGCSweep(TB_p bank)
 #endif
    bank->garbage_state =
       bank->garbage_state?TPIgnoreProps:TPGarbageFlag;
-
+   bank->recovered+=recovered;
    return recovered;
 }
 
@@ -1854,13 +2183,14 @@ Term_p TBCreateConstTerm(TB_p bank, FunCode fconst)
 Term_p TBCreateMinTerm(TB_p bank, FunCode min_const)
 {
    Type_p type = SigGetType(bank->sig, min_const);
-   long sort = GetReturnSort(type)->type_uid;
-   Term_p t = PDArrayElementP(bank->min_terms, sort);
+   assert(type);
+   Term_p t = PDArrayElementP(bank->min_terms, type->type_uid);
 
    if(!t)
    {
       t = TBCreateConstTerm(bank, min_const);
-      PDArrayAssignP(bank->min_terms, sort, t);
+      assert(t->type);
+      PDArrayAssignP(bank->min_terms, type->type_uid, t);
    }
    return t;
 }
@@ -1920,7 +2250,7 @@ Term_p TBFindRepr(TB_p bank, Term_p term)
    Term_p work;
    Term_p repr;
 
-   if (TermIsVar(term) || TermIsConst(term))
+   if (TermIsAnyVar(term) || TermIsConst(term))
    {
       return TBFind(bank, term);
    }
@@ -1970,7 +2300,7 @@ Term_p TBGetFirstConstTerm(TB_p bank, Type_p type)
 //   Find the best (according to is_better) constant of the give sort,
 //   and return a shared term with this constant. If no suitable
 //   constant exists, returns NULL. conj_dist_array contains number of
-//   occurances for each symbol in conjecture clauses, dist_array the
+//   occurrences for each symbol in conjecture clauses, dist_array the
 //   same for all clauses.
 //
 // Global Variables: -
@@ -2016,7 +2346,9 @@ Term_p TBGetFreqConstTerm(TB_p terms, Type_p type,
 //   it continues mapping t'. Else, it recursively applies f to
 //   arguments of t. Result term is guaranteed to be shared.
 //   Term mapper must also return shared term of the same type
-//   as the original one.
+//   as the original one. IMPORTANT: If f returns NULL this signifies
+//   that recursion should stop and the term is unaltered: this is used
+//   to painlessly implement optimizations.
 //
 // Global Variables: -
 //
@@ -2026,10 +2358,15 @@ Term_p TBGetFreqConstTerm(TB_p terms, Type_p type,
 
 Term_p TermMap(TB_p bank, Term_p t, TermMapper f)
 {
+   assert(t);
    Term_p s = f(bank, t);
-   assert(TermIsShared(s) && s->type == t->type);
+   assert(!s || (TermIsShared(s) && s->type == t->type));
 
-   if(s!=t)
+   if(!s)
+   {
+      s = t;
+   }
+   else if (s!=t)
    {
       s = TermMap(bank, s, f);
    }
@@ -2118,7 +2455,7 @@ Term_p ParseLet(Scanner_p in, TB_p bank)
    AcceptInpTok(in, Comma);
 
    SigEnterLetScope(bank->sig, type_decls);
-   Term_p body = TFormulaTPTPParse(in, bank);
+   Term_p body = TFormulaTSTPParse(in, bank);
    SigExitLetScope(bank->sig);
 
    Term_p let_term = make_let(bank, definitions, body);
@@ -2155,11 +2492,11 @@ Term_p ParseIte(Scanner_p in, TB_p bank)
 {
    AcceptInpTok(in, IteToken);
    AcceptInpTok(in, OpenBracket);
-   Term_p cond = TFormulaTPTPParse(in, bank);
+   Term_p cond = TFormulaTSTPParse(in, bank);
    AcceptInpTok(in, Comma);
-   Term_p if_true = TFormulaTPTPParse(in, bank);
+   Term_p if_true = TFormulaTSTPParse(in, bank);
    AcceptInpTok(in, Comma);
-   Term_p if_false = TFormulaTPTPParse(in, bank);
+   Term_p if_false = TFormulaTSTPParse(in, bank);
    AcceptInpTok(in, CloseBracket);
 
    Term_p res = TermTopAlloc(SIG_ITE_CODE, 3);
@@ -2168,12 +2505,51 @@ Term_p ParseIte(Scanner_p in, TB_p bank)
    res->args[2] = if_false;
 
    TermAssertSameSort(bank->sig, cond, bank->true_term);
-   TermAssertSameSort(bank->sig, if_false, if_true);
+   TermAssertSameSort(bank->sig, if_true, if_false);
 
    res->type = if_true->type;
    return TBTermTopInsert(bank, res);
 }
 
+/*-----------------------------------------------------------------------
+//
+// Function: NormalizePatternAppVar()
+//
+//   Tries to normalize applied variable so that all of its arguments
+//   are (?!?)
+//
+// Global Variables: -
+//
+// Side Effects    : -
+//
+/----------------------------------------------------------------------*/
+
+Term_p NormalizePatternAppVar(TB_p bank, Term_p s)
+{
+   if(TermIsFreeVar(s))
+   {
+      return s;
+   }
+
+   assert(TermIsAppliedFreeVar(s));
+   assert(bank);
+
+   s = LambdaEtaReduceDB(bank, s);
+   bool all_dbs = true;
+   for(long i=1; i<s->arity && all_dbs; i++)
+   {
+      all_dbs = TermIsDBVar(s->args[i]);
+   }
+
+   if(all_dbs && TermArrayNoDuplicates(s->args, s->arity))
+   {
+      return s;
+   }
+   else
+   {
+      return NULL;
+   }
+}
 
 
 /*---------------------------------------------------------------------*/
